% File automatically generated by PKG_ADD.sh
netcdf.getConstant = @netcdf_getConstant;
netcdf.getConstantNames = @netcdf_getConstantNames;
netcdf.inqLibVers = @netcdf_inqLibVers;
netcdf.setDefaultFormat = @netcdf_setDefaultFormat;
netcdf.setChunkCache = @netcdf_setChunkCache;
netcdf.getChunkCache = @netcdf_getChunkCache;
netcdf.create = @netcdf_create;
netcdf.open = @netcdf_open;
netcdf.abort = @netcdf_abort;
netcdf.sync = @netcdf_sync;
netcdf.setFill = @netcdf_setFill;
netcdf.inq = @netcdf_inq;
netcdf.inqUnlimDims = @netcdf_inqUnlimDims;
netcdf.inqFormat = @netcdf_inqFormat;
netcdf.defDim = @netcdf_defDim;
netcdf.renameDim = @netcdf_renameDim;
netcdf.defVar = @netcdf_defVar;
netcdf.renameVar = @netcdf_renameVar;
netcdf.defVarFill = @netcdf_defVarFill;
netcdf.inqVarFill = @netcdf_inqVarFill;
netcdf.defVarDeflate = @netcdf_defVarDeflate;
netcdf.inqVarDeflate = @netcdf_inqVarDeflate;
netcdf.defVarChunking = @netcdf_defVarChunking;
netcdf.inqVarChunking = @netcdf_inqVarChunking;
netcdf.defVarFletcher32 = @netcdf_defVarFletcher32;
netcdf.inqVarFletcher32 = @netcdf_inqVarFletcher32;
netcdf.endDef = @netcdf_endDef;
netcdf.reDef = @netcdf_reDef;
netcdf.putVar = @netcdf_putVar;
netcdf.getVar = @netcdf_getVar;
netcdf.close = @netcdf_close;
netcdf.inqAttName = @netcdf_inqAttName;
netcdf.inqAttID = @netcdf_inqAttID;
netcdf.inqAtt = @netcdf_inqAtt;
netcdf.getAtt = @netcdf_getAtt;
netcdf.putAtt = @netcdf_putAtt;
netcdf.copyAtt = @netcdf_copyAtt;
netcdf.renameAtt = @netcdf_renameAtt;
netcdf.delAtt = @netcdf_delAtt;
netcdf.inqVarID = @netcdf_inqVarID;
netcdf.inqVarIDs = @netcdf_inqVarIDs;
netcdf.inqVar = @netcdf_inqVar;
netcdf.inqDim = @netcdf_inqDim;
netcdf.inqDimID = @netcdf_inqDimID;
netcdf.inqDimIDs = @netcdf_inqDimIDs;
netcdf.defGrp = @netcdf_defGrp;
netcdf.inqGrps = @netcdf_inqGrps;
netcdf.inqGrpName = @netcdf_inqGrpName;
netcdf.inqGrpNameFull = @netcdf_inqGrpNameFull;
netcdf.inqGrpParent = @netcdf_inqGrpParent;
netcdf.inqGrpFullNcid = @netcdf_inqGrpFullNcid;
netcdf.inqNcid = @netcdf_inqNcid;
